@@ -483,11 +483,16 @@ tnkde <- function(lines, events, time_field, w, samples_loc, samples_time, kerne
   }else{
     if(adaptive_separate == TRUE){
       ## we want to use an adaptive bw in the network space
-      bws_net <- adaptive_bw(grid, events_loc, lines, bw_net, trim_bw_net, method,
+      bws_net_all <- adaptive_bw.mc(grid = grid,
+                             events = events_loc,
+                             lines = lines,
+                             bw = bw_net,
+                             trim_bw = trim_bw_net,
+                             method,
                              kernel_name, max_depth, tol, digits, sparse, verbose)
       ## and in the time space
       bws_time <- adaptive_bw_1d(events$time, w, bw_time, kernel_name)
-      bws_net <- bws_net[events$goid]
+      bws_net <- bws_net_all[events$goid]
     }else{
       interaction_bws <- adaptive_bw_tnkde(grid, events_loc, events, lines,
                                            bw_net, bw_time ,trim_bw_net, trim_bw_time,
@@ -511,9 +516,16 @@ tnkde <- function(lines, events, time_field, w, samples_loc, samples_time, kerne
 
     outside_mass_time <- correction_factor_time(events[[time_field]], samples_time, bws_time, kernel_name)
     corr_factor_net <- correction_factor(study_area,events_loc,lines,method, bws_net, kernel_name, tol, digits, max_depth, sparse)
-    # NOTE : we need to convert back the correction factor to mass outside network
-    outside_mass_net <- 1-(1/(corr_factor_net))
-    corr_factor <- 1/(1-(outside_mass_net * outside_mass_time))
+    corr_factor_time <- correction_factor_time(events[[time_field]], samples_time, bws_time, kernel_name)
+    # the final density is (dnet*corr_net * dtime*corr_time)*w
+    # because we only have multiplications, I can do the product here
+    # note : mutliplying the correction factor is equivalent to calculate the total inside mass
+    # and taking its inverse
+    corr_factor <- corr_factor_net * corr_factor_time
+
+    # OLD MASTER VERSION
+    # outside_mass_net <- 1-(1/(corr_factor_net))
+    # corr_factor <- 1/(1-(outside_mass_net * outside_mass_time))
 
   }else{
     corr_factor <- rep(1,nrow(events))
