@@ -37,7 +37,7 @@ test_that("Testing the bw selection function with CV likelihood and simple kerne
 
 
   #let us calculate the value with our function
-  obs_value <- bw_cv_likelihood_calc(bw_range = c(8,10),1,
+  obs_value <- bw_cv_likelihood_calc(bws = seq(8,10,1),
                                      lines = all_lines,
                                      events = event, w = c(1,1,1),
                                      check = F,
@@ -86,7 +86,7 @@ test_that("Testing the bw selection function with CV likelihood and simple kerne
 
 
   #let us calculate the value with our function
-  obs_value <- bw_cv_likelihood_calc(bw_range = c(6,8),1,
+  obs_value <- bw_cv_likelihood_calc(bws = seq(6,8,1),
                                      lines = all_lines,
                                      events = event, w = c(1,1,1),
                                      check = F,
@@ -134,7 +134,7 @@ test_that("Testing the bw selection function with CV likelihood and discontinuou
 
 
   #let us calculate the value with our function
-  obs_value <- bw_cv_likelihood_calc(c(8,10),1,
+  obs_value <- bw_cv_likelihood_calc(seq(8,10,1),
                     lines = all_lines,
                     events = event, w = c(1,1,1),
                     check = F,
@@ -184,7 +184,7 @@ test_that("Testing the bw selection function with CV likelihood and continuous k
 
 
   #let us calculate the value with our function
-  obs_value <- bw_cv_likelihood_calc(c(8,10),1,
+  obs_value <- bw_cv_likelihood_calc(seq(8,10,1),
                                      lines = all_lines,
                                      events = event, w = c(1,1,1),
                                      check = F,
@@ -258,8 +258,7 @@ test_that("Testing the bw selection function with CV likelihood and continuous k
 
 
   #let us calculate the value with our function
-  obs_value <- bw_cv_likelihood_calc(bw_range = c(11,12),
-                                           bw_step = 1,
+  obs_value <- bw_cv_likelihood_calc(bws = seq(11,12,1),
                                            lines = all_lines,
                                            events = event,
                                            w = c(1,1,1),
@@ -318,8 +317,7 @@ test_that("Testing the bw selection function with Van Lieshout's Criterion and s
 
 
   #let us calculate the value with our function
-  obs_value <- bw_cvl_calc(bw_range = c(8,10),
-                            bw_step = 1,
+  obs_value <- bw_cvl_calc(bws = seq(8,10,1),
                             lines = all_lines,
                             events = event,
                             w = c(1,1,1),
@@ -368,7 +366,7 @@ test_that("Testing the bw selection function with Van Lieshout's Criterion and d
 
 
   #let us calculate the value with our function
-  obs_value <- bw_cvl_calc(c(8,10),1,
+  obs_value <- bw_cvl_calc(seq(8,10,1),
                            lines = all_lines,
                            events = event, w = c(1,1,1),
                            check = F,
@@ -460,7 +458,7 @@ test_that("Testing the bw selection function with Van Lieshout's Criterion and c
 
 
   #let us calculate the value with our function
-  obs_value <- bw_cvl_calc(c(8,11),1,
+  obs_value <- bw_cvl_calc(seq(8,11,1),
                             lines = all_lines,
                             events = event, w = c(1,1,1),
                             check = F,
@@ -478,6 +476,63 @@ test_that("Testing the bw selection function with Van Lieshout's Criterion and c
 
 
 
+test_that("Testing the bw selection function with Van Lieshout's Criterion and simple kernel and adaptive", {
+
+  ## creating the simple situation
+  # start with de definition of some lines
+  wkt_lines <- c(
+    "LINESTRING (0 5, 0 0)",
+    "LINESTRING (-5 0, 0 0)",
+    "LINESTRING (0 -5, 0 0)",
+    "LINESTRING (5 0, 0 0)")
+
+  linesdf <- data.frame(wkt = wkt_lines,
+                        id = paste("l",1:length(wkt_lines),sep=""))
+
+  all_lines <- st_as_sf(linesdf, wkt = "wkt")
+
+  # definition of three events
+  event <- data.frame(x=c(0,3,0),
+                      y=c(3,0,-3),
+                      id = c(1,2,3))
+  event <- st_as_sf(event, coords = c("x","y"))
+
+
+  # we can admit a bw of 10 here
+  # the network distance between two points is 6
+  # so the density at one event is
+  s1 <- (quartic_kernel(6,10) + quartic_kernel(6,10) + quartic_kernel(0,10)) *(1/10)
+
+  hf0 <- c(s1, s1, s1)
+
+  h0 <- 10
+  gamma_val <- exp(sum(log(1/sqrt(hf0)))/3)
+  abws <- h0 * (1/sqrt(hf0)) * (1/gamma_val)
+
+  #so the score value is
+  Wl <- sum(as.numeric(st_length(all_lines)))
+  score <- ((1/s1+1/s1+1/s1) - Wl)**2
+
+
+  #let us calculate the value with our function
+  obs_value <- bw_cvl_calc(bws = seq(8,10,1),
+                           trim_bws = seq(8,10,1) * 2,
+                           lines = all_lines,
+                           events = event,
+                           w = c(1,1,1),
+                           check = F,
+                           kernel_name = "quartic",
+                           adaptive = TRUE,
+                           method = "simple",
+                           digits = 1,
+                           agg = NULL,
+                           verbose = F,
+                           tol = 0.00001
+  )
+  expect_equal(obs_value[3,2], score)
+})
+
+
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 #### Comparing multicore and single core ####
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -500,17 +555,27 @@ test_that("Testing that bw selection by cv-likelihood gives the same score in si
   ## multicore cv score
   future::plan(future::multisession(workers=1))
 
-  cv_scores.mc <- bw_cv_likelihood_calc.mc(c(200,400),100,
-                                 lines, events,
-                                 rep(1,nrow(events)),
-                                 "scaled gaussian", "discontinuous",
-                                 diggle_correction = FALSE, study_area = NULL,
-                                 max_depth = 8,
-                                 digits=2, tol=0.1, agg=5,
-                                 sparse=TRUE, grid_shape=c(2,2),
-                                 sub_sample = 1, verbose=FALSE, check=TRUE)
+  cv_scores.mc <- bw_cv_likelihood_calc.mc(
+    bws = seq(200,400,100),
+    lines = lines,
+    events = events,
+    w = rep(1,nrow(events)),
+    kernel_name = "scaled gaussian",
+    method = "discontinuous",
+    diggle_correction = FALSE,
+    study_area = NULL,
+    max_depth = 8,
+    digits=2,
+    tol=0.1,
+    agg=5,
+    sparse=TRUE,
+    grid_shape=c(2,2),
+    sub_sample = 1,
+    verbose=FALSE,
+    adaptive = FALSE,
+    check=TRUE)
 
-  cv_scores.mc2 <- bw_cv_likelihood_calc.mc(c(200,400),100,
+  cv_scores.mc2 <- bw_cv_likelihood_calc.mc(seq(200,400,100),
                                            lines, events,
                                            rep(1,nrow(events)),
                                            "scaled gaussian", "discontinuous",
@@ -521,8 +586,7 @@ test_that("Testing that bw selection by cv-likelihood gives the same score in si
                                            sub_sample = 1, verbose=TRUE, check=TRUE)
 
   ## single core cv score
-  cv_scores <- bw_cv_likelihood_calc(bw_range = c(200,400),
-                                     bw_step = 100,
+  cv_scores <- bw_cv_likelihood_calc(bws = seq(200,400,100),
                                      lines = lines,
                                      events = events,
                                      w = rep(1,nrow(events)),
@@ -563,7 +627,7 @@ test_that("Testing that bw selection with Van Lieshout's Criterion gives the sam
 
   ## multicore cv score
   future::plan(future::multisession(workers=1))
-  cv_scores.mc <- bw_cvl_calc.mc(c(200,400),100,
+  cv_scores.mc <- bw_cvl_calc.mc(seq(200,400,100),
                                            lines, events,
                                            rep(1,nrow(events)),
                                            "gaussian", "discontinuous",
@@ -573,7 +637,7 @@ test_that("Testing that bw selection with Van Lieshout's Criterion gives the sam
                                            sparse=TRUE, grid_shape=c(2,2),
                                            sub_sample = 1, verbose=FALSE, check=TRUE)
 
-  cv_scores.mc2 <- bw_cvl_calc.mc(c(200,400),100,
+  cv_scores.mc2 <- bw_cvl_calc.mc(seq(200,400,100),
                                  lines, events,
                                  rep(1,nrow(events)),
                                  "gaussian", "discontinuous",
@@ -581,10 +645,10 @@ test_that("Testing that bw selection with Van Lieshout's Criterion gives the sam
                                  max_depth = 8,
                                  digits=2, tol=0.1, agg=5,
                                  sparse=TRUE, grid_shape=c(2,2),
-                                 sub_sample = 1, verbose=FALSE, check=TRUE)
+                                 sub_sample = 1, verbose=TRUE, check=TRUE)
 
   ## single core cv score
-  cv_scores <- bw_cvl_calc(c(200,400),100,
+  cv_scores <- bw_cvl_calc(seq(200,400,100),
                                      lines, events,
                                      rep(1,nrow(events)),
                                      "gaussian", "discontinuous",
@@ -609,3 +673,173 @@ test_that("Testing that bw selection with Van Lieshout's Criterion gives the sam
 
 })
 
+
+#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+#### TEST FOR BW SELECTION WITH CV LIKELIHOOD AND ADAPTIVE BW ####
+#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+
+test_that("Testing the bw selection function with CV likelihood and discontinuous kernel and adaptive bw", {
+
+  ## creating the simple situation
+  # start with de definition of some lines
+  wkt_lines <- c(
+    "LINESTRING (0 5, 0 0)",
+    "LINESTRING (-5 0, 0 0)",
+    "LINESTRING (0 -5, 0 0)",
+    "LINESTRING (5 0, 0 0)")
+
+  linesdf <- data.frame(wkt = wkt_lines,
+                        id = paste("l",1:length(wkt_lines),sep=""))
+
+
+  all_lines <- st_as_sf(linesdf, wkt = "wkt")
+
+  # definition of three events
+  event <- data.frame(x=c(0,1,0),
+                      y=c(3,0,-3),
+                      id = c(1,2,3))
+  event <- st_as_sf(event, coords = c("x","y"))
+  # tm_shape(all_lines) + tm_lines('black') + tm_shape(event) + tm_dots('red',size = 1)
+  # we will use adaptive bandwidths and we will only test the result for a bw = 10
+
+  # for each point, we split the kernel at the intersection (1/3)
+
+  # we must first evaluate the density at each event
+  bw <- 10
+
+  # for point 1
+  dens1 <- (quartic_kernel(0,bw) + # self density
+    1/3 * quartic_kernel(4,bw) + # second point
+    1/3 * quartic_kernel(6,bw)) * (1/bw) # third point
+
+  # for point 2
+  dens2 <- (quartic_kernel(0,bw) + # self density
+              1/3 * quartic_kernel(4,bw) + # first point on top
+              1/3 * quartic_kernel(4,bw)) * (1/bw) # third point
+
+  # for point 3
+  dens3 <- dens1
+
+  # we can now calculate the local bandwidths
+  k <- c(dens1, dens2, dens3)
+  delta <- calc_gamma(k)
+  bws <- bw * (k**(-1/2) * delta**(-1))
+
+  # we can now calculate the loo of each point
+  # for point 1
+  loo1 <- (((1/3) * quartic_kernel(4,bws[[2]]))  * (1/bws[[2]]))  + # density from point 2
+    (((1/3) * quartic_kernel(6,bws[[3]]))  * (1/bws[[3]])) # density from point 3
+
+  loo2 <- ((1/3) * quartic_kernel(4,bws[[1]]) * (1/bws[[1]]) +
+             (1/3) * quartic_kernel(4,bws[[3]]) * (1/bws[[3]]))
+
+  loo3 <- loo1
+
+  #so the CV likelihood is the sum for each point loo
+  #in other words, three times the sum of two kerneffect
+  loov <- c(loo1, loo2, loo3)
+  total <- sum(log(loov)) / 3
+
+
+  #let us calculate the value with our function
+  obs_value <- bw_cv_likelihood_calc(seq(8,10,1),
+                                     lines = all_lines,
+                                     events = event, w = c(1,1,1),
+                                     check = F,
+                                     kernel_name = "quartic",
+                                     method = "discontinuous",
+                                     adaptive = TRUE,
+                                     trim_bws = c(50,50,50),
+                                     digits = 1,
+                                     agg = NULL,
+                                     verbose = T,
+                                     grid_shape = c(3,3),
+                                     tol = 0.00001
+  )
+  expect_equal(obs_value[3,2], total)
+})
+
+
+test_that("Testing the bw selection function with CV likelihood and discontinuous kernel and adaptive bw with weights", {
+
+  ## creating the simple situation
+  # start with de definition of some lines
+  wkt_lines <- c(
+    "LINESTRING (0 5, 0 0)",
+    "LINESTRING (-5 0, 0 0)",
+    "LINESTRING (0 -5, 0 0)",
+    "LINESTRING (5 0, 0 0)")
+
+  linesdf <- data.frame(wkt = wkt_lines,
+                        id = paste("l",1:length(wkt_lines),sep=""))
+
+
+  all_lines <- st_as_sf(linesdf, wkt = "wkt")
+
+  # definition of three events
+  event <- data.frame(x=c(0,1,0),
+                      y=c(3,0,-3),
+                      id = c(1,2,3),
+                      w = c(2,1,1))
+
+  event <- st_as_sf(event, coords = c("x","y"))
+  # tm_shape(all_lines) + tm_lines('black') + tm_shape(event) + tm_dots('red',size = 1)
+  # we will use adaptive bandwidths and we will only test the result for a bw = 10
+
+  # for each point, we split the kernel at the intersection (1/3)
+
+  # we must first evaluate the density at each event
+  bw <- 10
+
+  # for point 1
+  dens1 <- (quartic_kernel(0,bw) * 2 + # self density
+              1/3 * quartic_kernel(4,bw) + # first point on the left
+              1/3 * quartic_kernel(6,bw)) * (1/bw) # second point
+
+  # for point 2
+  dens2 <- (quartic_kernel(0,bw) + # self density
+              1/3 * quartic_kernel(4,bw) * 2 + # first point on top
+              1/3 * quartic_kernel(4,bw)) * (1/bw) # second point below
+
+  # for point 3
+  dens3 <- (quartic_kernel(0,bw) + # self density
+              1/3 * quartic_kernel(4,bw) + # first point on the left
+              1/3 * quartic_kernel(6,bw) * 2) * (1/bw) # second point
+
+  # we can now calculate the local bandwidths
+  k <- c(dens1, dens2, dens3)
+  delta <- calc_gamma(k)
+  bws <- bw * (k**(-1/2) * delta**(-1))
+
+  # we can now calculate the loo of each point
+  loo1 <- ((1/3) * quartic_kernel(4,bws[[2]]) * (1/bws[[2]]) +
+             (1/3) * quartic_kernel(6,bws[[3]]) * (1/bws[[3]]))
+
+  loo2 <- ((1/3) * quartic_kernel(4,bws[[1]]) * 2 * (1/bws[[1]]) +
+             (1/3) * quartic_kernel(4,bws[[3]]) * (1/bws[[3]]))
+
+  loo3 <- ((1/3) * quartic_kernel(4,bws[[2]]) * (1/bws[[2]]) +
+             (1/3) * quartic_kernel(6,bws[[1]]) * 2 * (1/bws[[1]]))
+
+  #so the CV likelihood is the sum for each point loo
+  #in other words, three times the sum of two kerneffect
+  loov <- c(loo1, loo2, loo3)
+  total <- sum(log(loov)) / 3
+
+
+  #let us calculate the value with our function
+  obs_value <- bw_cv_likelihood_calc(seq(8,10,1),
+                                     lines = all_lines,
+                                     events = event, w = event$w,
+                                     check = F,
+                                     kernel_name = "quartic",
+                                     method = "discontinuous",
+                                     adaptive = TRUE,
+                                     trim_bws = c(50,50,50),
+                                     digits = 1,
+                                     agg = NULL, verbose = F,tol = 0.00001
+  )
+  expect_equal(obs_value[3,2], total)
+})
+
